package com.exner.tools.immichuploader

import android.content.Context
import androidx.room.Room
import com.exner.tools.immichuploader.data.MyDAO
import com.exner.tools.immichuploader.data.MyDatabase
import com.exner.tools.immichuploader.json.adapters.InstantJsonAdapter
import com.exner.tools.immichuploader.json.adapters.LocalDateTimeJsonAdapter
import com.exner.tools.immichuploader.network.ImmichApi
import com.exner.tools.immichuploader.network.ImmichRepository
import com.exner.tools.immichuploader.network.ImmichRepositoryImpl
import com.squareup.moshi.Moshi
import dagger.Module
import dagger.Provides
import dagger.hilt.InstallIn
import dagger.hilt.android.qualifiers.ApplicationContext
import dagger.hilt.components.SingletonComponent
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.moshi.MoshiConverterFactory
import retrofit2.converter.scalars.ScalarsConverterFactory
import javax.inject.Provider
import javax.inject.Singleton

@Module
@InstallIn(SingletonComponent::class)
object AppComponent {

    @Singleton
    @Provides
    fun provideDAO(myDatabase: MyDatabase): MyDAO = myDatabase.dataDAO()

    @Singleton
    @Provides
    fun provideDatabase(
        @ApplicationContext context: Context,
        provider: Provider<MyDAO>
    ): MyDatabase = Room.databaseBuilder(
        context = context.applicationContext,
        MyDatabase::class.java,
        name = "asset_database"
    ).fallbackToDestructiveMigration(dropAllTables = true).build()

    @Singleton
    @Provides
    fun provideImmichApi(retrofit: Retrofit): ImmichApi {
        return retrofit.create(ImmichApi::class.java)
    }

    @Singleton
    @Provides
    fun provideImmichRepository(immichApi: ImmichApi): ImmichRepository {
        return ImmichRepositoryImpl(immichApi)
    }

    @Singleton
    @Provides
    fun provideRetrofit(okHttpClient: OkHttpClient): Retrofit {
        val moshi: Moshi = Moshi.Builder()
            .add(InstantJsonAdapter())
            .add(LocalDateTimeJsonAdapter())
            .build()
        val logging = HttpLoggingInterceptor()
        // set your desired log level
        logging.setLevel(HttpLoggingInterceptor.Level.BASIC)
        val httpClient = OkHttpClient.Builder()
        // add your other interceptors …
        // add logging as last interceptor
        httpClient.addInterceptor(logging)

        return Retrofit.Builder()
            .baseUrl("http://my.immich.server:2283/")
            .client(okHttpClient)
            .addConverterFactory(ScalarsConverterFactory.create())
            .addConverterFactory(MoshiConverterFactory.create(moshi))
            .client(httpClient.build())
            .build()
    }

    @Singleton
    @Provides
    fun providesOkHttpClient(): OkHttpClient {
        return OkHttpClient.Builder()
            .build()
    }

}