package com.exner.tools.immichuploader

import android.os.Bundle
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.work.WorkManager
import com.exner.tools.immichuploader.ui.theme.ImmichUploaderTheme
import com.ramcosta.composedestinations.DestinationsNavHost
import com.ramcosta.composedestinations.generated.NavGraphs
import com.ramcosta.composedestinations.rememberNavHostEngine
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        enableEdgeToEdge()
        setContent {
            ImmichUploaderTheme {
                val engine = rememberNavHostEngine()
                val navController = engine.rememberNavController()

                DestinationsNavHost(
                    navController = navController,
                    navGraph = NavGraphs.root,
                )
            }
        }
    }

    override fun onPause() {
        super.onPause()

        // kill all workers (until I find something better to do)
        val workManager = WorkManager.getInstance(this)
        Log.d("MainActivity", "Cancelling 'old' workers...")
        workManager.cancelUniqueWork("ImmichUploaderDiscoveryWorker")
        workManager.cancelUniqueWork("ImmichUploaderUploadWorker")
        // DEBUG
        workManager.cancelAllWork()
    }
}
