package com.exner.tools.immichuploader

import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import androidx.core.content.ContextCompat

class Permissions(
    val context: Context
) {
    data class IndividualPermission(
        val name: String,
        val minSdkVersion: Int,
        val maxSdkVersion: Int? = null
    )

    val MINSDKVERSION = 29

    val allNecessaryPermissions = listOf(
        IndividualPermission(
            name = "android.permission.READ_EXTERNAL_STORAGE",
            minSdkVersion = MINSDKVERSION,
            maxSdkVersion = 32
        ),
        IndividualPermission(
            name = "android.permission.READ_MEDIA_IMAGES",
            minSdkVersion = 33,
            maxSdkVersion = 36
        ),
        IndividualPermission(
            name = "android.permission.READ_MEDIA_VIDEO",
            minSdkVersion = 33,
            maxSdkVersion = 36
        ),
        IndividualPermission(
            name = "android.permission.ACCESS_MEDIA_LOCATION",
            minSdkVersion = MINSDKVERSION,
            maxSdkVersion = 36
        ),
    )

    private val currentReleaseLevel: Int = Build.VERSION.SDK_INT

    private val missingPermissions: MutableList<IndividualPermission> = mutableListOf()

    private val allNecessaryPermissionsAsListOfStrings: MutableList<String> = mutableListOf()

    // check all the permissions we need
    init {
        checkNecessaryPermissions()
    }

    fun checkNecessaryPermissions() {
        allNecessaryPermissions.forEach { individualPermission ->
            if (individualPermission.minSdkVersion <= currentReleaseLevel && (individualPermission.maxSdkVersion == null || individualPermission.maxSdkVersion >= currentReleaseLevel)) {
                allNecessaryPermissionsAsListOfStrings.add(individualPermission.name)
                val checkResult =
                    ContextCompat.checkSelfPermission(context, individualPermission.name)
//                Log.d("PERMISSIONS", "Permission ${individualPermission.name} is $checkResult")
                if (checkResult == PackageManager.PERMISSION_DENIED) {
                    missingPermissions.add(individualPermission)
                } else if (checkResult == PackageManager.PERMISSION_GRANTED) {
                    missingPermissions.remove(individualPermission)
                }
            }
        }
    }

    fun getAllNecessaryPermissionsAsListOfStrings(): List<String> {
        return allNecessaryPermissionsAsListOfStrings
    }

    fun doWeHaveAllPermissions(): Boolean {
        return (missingPermissions.isEmpty())
    }

    fun getMissingPermissions(): List<IndividualPermission> {
        return missingPermissions
    }

}