package com.exner.tools.immichuploader.data

import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Insert
import androidx.room.Query
import androidx.room.Update
import com.exner.tools.immichuploader.data.entities.FileDeletionCache
import com.exner.tools.immichuploader.data.entities.MediaFile
import kotlinx.coroutines.flow.Flow

@Dao
interface MyDAO {
    //
    // observers
    //
    @Query("SELECT * FROM mediafile ORDER BY created_date DESC")
    fun observeMediaFiles(): Flow<List<MediaFile>>

    @Query("SELECT * FROM mediafile WHERE last_backup IS NULL OR backup_status IS NULL OR immich_asset_id IS NULL ORDER BY created_date DESC")
    fun observeMediaFilesNotBackedUp(): Flow<List<MediaFile>>

    @Query("SELECT * FROM filedeletioncache")
    fun observeFileDeletionCache(): Flow<List<FileDeletionCache>>

    //
    // others
    //
    @Query("SELECT COUNT(uri) FROM mediafile")
    fun countMediaFiles(): Flow<Int>

    @Query("SELECT COUNT(uri) FROM mediafile WHERE last_backup IS NULL")
    fun countMediaFilesNotBackedUp(): Flow<Int>

    @Query("SELECT * FROM mediafile")
    suspend fun getAllMediaFiles(): List<MediaFile>

    @Query("SELECT * FROM mediafile WHERE last_backup IS NULL")
    suspend fun getAllMediaFilesNotBackedUp(): List<MediaFile>

    @Query("SELECT COUNT(uri_string) FROM filedeletioncache")
    fun countFilesToDelete(): Flow<Int>

    @Query("SELECT * FROM filedeletioncache")
    suspend fun getAllFilesToDelete(): List<FileDeletionCache>

    //
    // CRUD
    //

    @Query("SELECT * FROM mediafile WHERE uri=:uri")
    suspend fun getMediaFile(uri: String): MediaFile?

    @Insert
    suspend fun insertMediaFile(mediaFile: MediaFile): Long

    @Update
    suspend fun updateMediaFile(mediaFile: MediaFile)

    @Query("SELECT * FROM filedeletioncache where uri_string=:uriString")
    suspend fun getFileToDelete(uriString: String): FileDeletionCache?

    @Insert
    suspend fun insertFileDeletionCache(file: FileDeletionCache)

    @Update
    suspend fun updateFileToDelete(file: FileDeletionCache)

    @Delete
    suspend fun deleteFileDeletionCache(file: FileDeletionCache)

    @Query("UPDATE filedeletioncache SET `delete`=:delete")
    suspend fun setDeleteFlagForAllInDeletionCache(delete: Boolean)
}