package com.exner.tools.immichuploader.data

import androidx.annotation.WorkerThread
import com.exner.tools.immichuploader.data.entities.FileDeletionCache
import com.exner.tools.immichuploader.data.entities.MediaFile
import kotlinx.coroutines.flow.Flow
import javax.inject.Inject

class MyRepository @Inject constructor(private val myDAO: MyDAO) {
    //
    // observers
    //
    val observerMediaFiles: Flow<List<MediaFile>> = myDAO.observeMediaFiles()
    val observeMediaFilesNotBackedUp: Flow<List<MediaFile>> = myDAO.observeMediaFilesNotBackedUp()
    val observeFilesToDelete: Flow<List<FileDeletionCache>> = myDAO.observeFileDeletionCache()

    //
    // others
    //
    val countMediaFiles: Flow<Int> = myDAO.countMediaFiles()
    val countMediaFilesNotBackedUp: Flow<Int> = myDAO.countMediaFilesNotBackedUp()
    val countFilesToDelete: Flow<Int> = myDAO.countFilesToDelete()

    @WorkerThread
    suspend fun getAllMediaFiles(): List<MediaFile> {
        return myDAO.getAllMediaFiles()
    }

    @WorkerThread
    suspend fun getAllMediaFilesNotBackedUp(): List<MediaFile> {
        return myDAO.getAllMediaFilesNotBackedUp()
    }

    @WorkerThread
    suspend fun getAllFilesToDelete(): List<FileDeletionCache> {
        return myDAO.getAllFilesToDelete()
    }

    //
    // CRUD
    //

    @WorkerThread
    suspend fun getMediaFile(uri: String): MediaFile? {
        return myDAO.getMediaFile(uri)
    }

    @WorkerThread
    suspend fun insertMediaFile(mediaFile: MediaFile) {
        myDAO.insertMediaFile(mediaFile)
    }

    @WorkerThread
    suspend fun updateMediaFile(mediaFile: MediaFile) {
        myDAO.updateMediaFile(mediaFile)
    }

    @WorkerThread
    suspend fun getFileToDelete(fileUriString: String): FileDeletionCache? {
        return myDAO.getFileToDelete(fileUriString)
    }

    @WorkerThread
    suspend fun insertFileToDelete(file: FileDeletionCache) {
        myDAO.insertFileDeletionCache(file)
    }

    @WorkerThread
    suspend fun updateFileToDelete(file: FileDeletionCache) {
        myDAO.updateFileToDelete(file)
    }

    @WorkerThread
    suspend fun deleteFileToDelete(file: FileDeletionCache) {
        myDAO.deleteFileDeletionCache(file)
    }

    @WorkerThread
    suspend fun setDeleteFlagForAllInDeletionCache(delete: Boolean) {
        myDAO.setDeleteFlagForAllInDeletionCache(delete)
    }
}
