package com.exner.tools.immichuploader.network

import android.content.Context
import android.util.Log
import com.exner.tools.immichuploader.data.entities.MediaFile
import com.exner.tools.immichuploader.network.models.ImmichUploadResult
import java.io.IOException

private const val TAG = "IAManager"

class ImmichApiManager(
    val context: Context,
    private val immichRepository: ImmichRepository
) {
    fun uploadAsset(
        url: String,
        apiKey: String,
        mediaFile: MediaFile,
        deviceId: String,
    ): ImmichUploadResult {
        try {
            val response = immichRepository.uploadAsset(
                url,
                apiKey,
                mediaFile,
                deviceId,
                context
            ).execute()
            if (response.code() == 200 || response.code() == 201) {
                // record this successful upload
                val responseBody = response.body()
                if (responseBody?.id != null && responseBody.status != null) {
                    return ImmichUploadResult.ImmichUploadSuccess(
                        id = responseBody.id,
                        status = responseBody.status
                    )
                }
            } else {
                Log.i(TAG, "this is an error: ${response.message()}")
                return ImmichUploadResult.ImmichUploadFailure(response.message())
            }
        } catch (ioe: IOException) {
            Log.i(TAG, "IOException: ${ioe.message}")
            Log.i(TAG, "IOException: ${ioe.cause}")
            return ImmichUploadResult.ImmichUploadFailure(ioe.message)
        }
        return ImmichUploadResult.ImmichUploadFailure("Unknown")
    }
}
