package com.exner.tools.immichuploader.network

import android.content.Context
import androidx.core.net.toUri
import com.exner.tools.immichuploader.data.entities.MediaFile
import com.exner.tools.immichuploader.network.models.ImmichUploadAssetResponse
import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.TimeZone
import kotlinx.datetime.format
import kotlinx.datetime.format.FormatStringsInDatetimeFormats
import kotlinx.datetime.format.byUnicodePattern
import kotlinx.datetime.toLocalDateTime
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.MultipartBody
import okhttp3.RequestBody
import okhttp3.RequestBody.Companion.toRequestBody
import retrofit2.Call

class ImmichRepositoryImpl(
    private val immichApi: ImmichApi
) : ImmichRepository {

    @OptIn(FormatStringsInDatetimeFormats::class)
    override fun uploadAsset(
        url: String,
        apiKey: String,
        mediaFile: MediaFile,
        deviceId: String,
        context: Context
    ): Call<ImmichUploadAssetResponse> {
        val fileCreatedLDT = mediaFile.createdDate?.toLocalDateTime(TimeZone.currentSystemDefault())
        val fileModifiedLDT =
            mediaFile.updatedDate?.toLocalDateTime(TimeZone.currentSystemDefault())
        val dateTimeFormat = LocalDateTime.Format {
            byUnicodePattern("yyyy-MM-dd HH:mm:ss")
        }
        // create multipart part map
        val partMap: MutableMap<String, RequestBody> = mutableMapOf()
        val contentTypeString =
            context.contentResolver.getType(mediaFile.uri.toUri())
        val contentType = contentTypeString!!.toMediaTypeOrNull()
        val contentPart =
            InputStreamRequestBody(contentType!!, context.contentResolver, mediaFile.uri.toUri())
        val requestBody = MultipartBody.Builder()
            .setType(MultipartBody.FORM)
            .addFormDataPart("assetData", mediaFile.name, contentPart)
            .build()
        partMap.put(
            "deviceAssetId",
            mediaFile.deviceAssetId.toString().toRequestBody("text/plain".toMediaTypeOrNull())
        )
        partMap.put("deviceId", deviceId.toRequestBody("text/plain".toMediaTypeOrNull()))
        partMap.put(
            "fileCreatedAt",
            fileCreatedLDT?.format(dateTimeFormat).toString().toRequestBody(
                "text/plain".toMediaTypeOrNull()
            )
        )
        partMap.put(
            "fileModifiedAt",
            fileModifiedLDT?.format(dateTimeFormat).toString().toRequestBody(
                "text/plain".toMediaTypeOrNull()
            )
        )
        return immichApi.uploadAsset(
            url = url,
            apiKey = apiKey,
            partMap = partMap,
            mediaFile = requestBody.part(0),
        )
    }
}