package com.exner.tools.immichuploader.work

import android.app.PendingIntent
import android.app.RecoverableSecurityException
import android.content.Context
import android.net.Uri
import android.os.Build
import android.provider.MediaStore
import android.util.Log
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.IntentSenderRequest
import androidx.annotation.RequiresApi
import androidx.core.net.toUri
import com.exner.tools.immichuploader.data.MyRepository
import com.exner.tools.immichuploader.data.entities.FileDeletionCache

private const val TAG = "DI"

suspend fun deleteImage(
    launcher: ActivityResultLauncher<IntentSenderRequest>,
    context: Context,
    repository: MyRepository,
    fileToDelete: FileDeletionCache
) {
    val uri = fileToDelete.uriString.toUri()
    val resolver = context.contentResolver

    // mark it deleted no matter what
    repository.deleteFileToDelete(fileToDelete)

    try {
        Log.d(TAG, "Trying to delete file $uri the easy way...")
        resolver.delete(uri, null, null)
    } catch (securityException: SecurityException) {
        Log.d(TAG, "Nope, not that easy.")
        var pendingIntent: PendingIntent? = null

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            val uriList: ArrayList<Uri> = arrayListOf(uri)
            Log.d(TAG, "Trying to delete file $uri using MediaStore request...")
            pendingIntent = MediaStore.createDeleteRequest(resolver, uriList)
        } else if (securityException is RecoverableSecurityException) {
            val recoverableSecurityException: RecoverableSecurityException =
                securityException
            Log.d(TAG, "Trying asking the user...")
            pendingIntent = recoverableSecurityException.userAction.actionIntent
        }
        if (pendingIntent != null) {
            val senderRequest = IntentSenderRequest.Builder(pendingIntent).build()
            launcher.launch(senderRequest)
        }
    }
}

@RequiresApi(Build.VERSION_CODES.R)
suspend fun deleteImagesAboveAPI30(
    launcher: ActivityResultLauncher<IntentSenderRequest>,
    context: Context,
    repository: MyRepository,
    filesToDelete: List<FileDeletionCache>
) {
    val resolver = context.contentResolver

    val uriList = filesToDelete.map { fileToDelete ->
        // mark it deleted no matter what
        repository.deleteFileToDelete(fileToDelete)
        val uri = fileToDelete.uriString.toUri()
        Log.d(TAG, "Queueing $uri for deletion")
        uri
    }

    if (uriList.isNotEmpty()) {
        // some files we were not able to delete the simple way, so let's go ask
        var pendingIntent: PendingIntent? = null

        Log.d(TAG, "Trying to trash ${uriList.size} files using MediaStore request...")
        pendingIntent = MediaStore.createTrashRequest(resolver, uriList, true)

        val senderRequest = IntentSenderRequest.Builder(pendingIntent).build()
        launcher.launch(senderRequest)
    }
}
