package com.exner.tools.immichuploader.work

import android.annotation.SuppressLint
import android.content.Context
import android.provider.Settings
import android.util.Log
import androidx.hilt.work.HiltWorker
import androidx.work.CoroutineWorker
import androidx.work.WorkerParameters
import androidx.work.workDataOf
import com.exner.tools.immichuploader.data.MyRepository
import com.exner.tools.immichuploader.network.ImmichApiManager
import com.exner.tools.immichuploader.network.ImmichRepository
import com.exner.tools.immichuploader.preferences.UserPreferencesRepository
import dagger.assisted.Assisted
import dagger.assisted.AssistedInject
import kotlinx.coroutines.flow.firstOrNull

private const val TAG = "USMAWorker"

@HiltWorker
class UploadSingleMediaAssetWorker @AssistedInject constructor(
    @Assisted val context: Context,
    @Assisted workerParams: WorkerParameters,
    private val userPreferencesRepository: UserPreferencesRepository,
    private val repository: MyRepository,
    val immichRepository: ImmichRepository,
) : CoroutineWorker(context, workerParams) {

    @SuppressLint("HardwareIds")
    override suspend fun doWork(): Result {
        // get input data
        val assetUri = inputData.getString(KEY_MEDIA_URI_STRING)
        val immichServerUrlText = inputData.getString(KEY_IMMICH_SERVER_URL_TEXT)
        val immichApiKey = inputData.getString(KEY_IMMICH_API_KEY)

        setProgress(workDataOf(PROGRESS to 0, CURRENT_ASSET to null, STATUS to null))
        // other stuff we need
        val deviceId =
            Settings.Secure.getString(
                applicationContext.contentResolver,
                Settings.Secure.ANDROID_ID
            )

        val imageCategoriesThatShouldBeDeletedAfterSuccessfulUpload =
            userPreferencesRepository.deleteMediaAfterSuccessfulUploadPerPseudoFolder()
                .firstOrNull()

        val immichApiManager =
            ImmichApiManager(context = applicationContext, immichRepository)

        Log.d(TAG, "U 1 file to backup...")

        if (immichServerUrlText != null && immichApiKey != null && !isStopped) {
            // look up the MediaFile in the repository
            val mediaFile = repository.getMediaFile(assetUri!!)

            if (mediaFile != null) {
                uploadAsset(
                    repository,
                    1,
                    1,
                    mediaFile,
                    immichApiManager,
                    immichServerUrlText,
                    immichApiKey,
                    deviceId,
                    imageCategoriesThatShouldBeDeletedAfterSuccessfulUpload,
                    this
                )
            }
        }

        Log.d(TAG, "U 1 file backed up.")

        // nothing to return
        return Result.success()
    }

    companion object {
        const val PROGRESS = "Progress"
        const val CURRENT_ASSET = "Asset"
        const val STATUS = "Status"
        const val KEY_MEDIA_URI_STRING = "KEY_MEDIA_URI_STRING"
        const val KEY_IMMICH_SERVER_URL_TEXT = "KEY_IMMICH_SERVER_URL_TEXT"
        const val KEY_IMMICH_API_KEY = "KEY_IMMICH_API_KEY"
    }
}